@@ -15,22 +15,17 @@
  *                                                                         *
  *   You should have received a copy of the GNU Lesser General Public      *
  *   License along with this library; if not, write to the Free Software   *
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
- *   USA                                                                   *
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
+ *   02110-1301  USA                                                       *
  *                                                                         *
  *   Alternatively, this file is available under the Mozilla Public        *
  *   License Version 1.1.  You may obtain a copy of the License at         *
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef WITH_MP4
-
 #include <tdebug.h>
 #include <tstring.h>
+#include <tpropertymap.h>
 #include "mp4atom.h"
 #include "mp4tag.h"
 #include "mp4file.h"
@@ -69,7 +64,16 @@ MP4::File::File(FileName file, bool readProperties, AudioProperties::ReadStyle a
     : TagLib::File(file)
 {
   d = new FilePrivate;
-  read(readProperties, audioPropertiesStyle);
+  if(isOpen())
+    read(readProperties, audioPropertiesStyle);
+}
+
+MP4::File::File(IOStream *stream, bool readProperties, AudioProperties::ReadStyle audioPropertiesStyle)
+    : TagLib::File(stream)
+{
+  d = new FilePrivate;
+  if(isOpen())
+    read(readProperties, audioPropertiesStyle);
 }
 
 MP4::File::~File()
@@ -83,6 +87,21 @@ MP4::File::tag() const
   return d->tag;
 }
 
+PropertyMap MP4::File::properties() const
+{
+  return d->tag->properties();
+}
+
+void MP4::File::removeUnsupportedProperties(const StringList &properties)
+{
+  d->tag->removeUnsupportedProperties(properties);
+}
+
+PropertyMap MP4::File::setProperties(const PropertyMap &properties)
+{
+  return d->tag->setProperties(properties);
+}
+
 MP4::Properties *
 MP4::File::audioProperties() const
 {
@@ -129,10 +148,16 @@ MP4::File::read(bool readProperties, Properties::ReadStyle audioPropertiesStyle)
 bool
 MP4::File::save()
 {
-  if(!isValid())
+  if(readOnly()) {
+    debug("MP4::File::save() -- File is read only.");
     return false;
+  }
+
+  if(!isValid()) {
+    debug("MP4::File::save() -- Trying to save invalid file.");
+    return false;
+  }
 
   return d->tag->save();
 }
 
-#endif
