@@ -15,22 +15,17 @@
  *                                                                         *
  *   You should have received a copy of the GNU Lesser General Public      *
  *   License along with this library; if not, write to the Free Software   *
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
- *   USA                                                                   *
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
+ *   02110-1301  USA                                                       *
  *                                                                         *
  *   Alternatively, this file is available under the Mozilla Public        *
  *   License Version 1.1.  You may obtain a copy of the License at         *
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef WITH_MP4
-
 #include <taglib.h>
 #include <tdebug.h>
+#include "trefcounter.h"
 #include "mp4item.h"
 
 using namespace TagLib;
@@ -38,15 +33,20 @@ using namespace TagLib;
 class MP4::Item::ItemPrivate : public RefCounter
 {
 public:
-  ItemPrivate() : RefCounter(), valid(true) {}
+  ItemPrivate() : RefCounter(), valid(true), atomDataType(TypeUndefined) {}
 
   bool valid;
+  AtomDataType atomDataType;
   union {
     bool m_bool;
     int m_int;
     IntPair m_intPair;
+    uchar m_byte;
+    uint m_uint;
+    long long m_longlong;
   };
   StringList m_stringList;
+  ByteVectorList m_byteVectorList;
   MP4::CoverArtList m_coverArtList;
 };
 
@@ -91,6 +91,24 @@ MP4::Item::Item(int value)
   d->m_int = value;
 }
 
+MP4::Item::Item(uchar value)
+{
+  d = new ItemPrivate;
+  d->m_byte = value;
+}
+
+MP4::Item::Item(uint value)
+{
+  d = new ItemPrivate;
+  d->m_uint = value;
+}
+
+MP4::Item::Item(long long value)
+{
+  d = new ItemPrivate;
+  d->m_longlong = value;
+}
+
 MP4::Item::Item(int value1, int value2)
 {
   d = new ItemPrivate;
@@ -98,6 +116,12 @@ MP4::Item::Item(int value1, int value2)
   d->m_intPair.second = value2;
 }
 
+MP4::Item::Item(const ByteVectorList &value)
+{
+  d = new ItemPrivate;
+  d->m_byteVectorList = value;
+}
+
 MP4::Item::Item(const StringList &value)
 {
   d = new ItemPrivate;
@@ -110,6 +134,16 @@ MP4::Item::Item(const MP4::CoverArtList &value)
   d->m_coverArtList = value;
 }
 
+void MP4::Item::setAtomDataType(MP4::AtomDataType type)
+{
+  d->atomDataType = type;
+}
+
+MP4::AtomDataType MP4::Item::atomDataType() const
+{
+  return d->atomDataType;
+}
+
 bool
 MP4::Item::toBool() const
 {
@@ -122,6 +156,24 @@ MP4::Item::toInt() const
   return d->m_int;
 }
 
+uchar
+MP4::Item::toByte() const
+{
+  return d->m_byte;
+}
+
+TagLib::uint
+MP4::Item::toUInt() const
+{
+  return d->m_uint;
+}
+
+long long
+MP4::Item::toLongLong() const
+{
+  return d->m_longlong;
+}
+
 MP4::Item::IntPair
 MP4::Item::toIntPair() const
 {
@@ -134,6 +186,12 @@ MP4::Item::toStringList() const
   return d->m_stringList;
 }
 
+ByteVectorList
+MP4::Item::toByteVectorList() const
+{
+  return d->m_byteVectorList;
+}
+
 MP4::CoverArtList
 MP4::Item::toCoverArtList() const
 {
@@ -146,4 +204,3 @@ MP4::Item::isValid() const
   return d->valid;
 }
 
-#endif
