@@ -23,12 +23,6 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef WITH_MP4
-
 #include <tdebug.h>
 #include <tstring.h>
 #include "mp4file.h"
@@ -40,13 +34,15 @@ using namespace TagLib;
 class MP4::Properties::PropertiesPrivate
 {
 public:
-  PropertiesPrivate() : length(0), bitrate(0), sampleRate(0), channels(0), bitsPerSample(0) {}
+  PropertiesPrivate() : length(0), bitrate(0), sampleRate(0), channels(0), bitsPerSample(0), encrypted(false), codec(MP4::Properties::Unknown) {}
 
   int length;
   int bitrate;
   int sampleRate;
   int channels;
   int bitsPerSample;
+  bool encrypted;
+  Codec codec;
 };
 
 MP4::Properties::Properties(File *file, MP4::Atoms *atoms, ReadStyle style)
@@ -91,15 +87,24 @@ MP4::Properties::Properties(File *file, MP4::Atoms *atoms, ReadStyle style)
 
   file->seek(mdhd->offset);
   data = file->readBlock(mdhd->length);
-  if(data[8] == 0) {
-    unsigned int unit = data.mid(20, 4).toUInt();
-    unsigned int length = data.mid(24, 4).toUInt();
-    d->length = length / unit;
+  uint version = data[8];
+  if(version == 1) {
+    if (data.size() < 36 + 8) {
+      debug("MP4: Atom 'trak.mdia.mdhd' is smaller than expected");
+      return;
+    }
+    const long long unit   = data.toLongLong(28U);
+    const long long length = data.toLongLong(36U);
+    d->length = unit ? int(length / unit) : 0;
   }
   else {
-    long long unit = data.mid(28, 8).toLongLong();
-    long long length = data.mid(36, 8).toLongLong();
-    d->length = int(length / unit);
+    if (data.size() < 24 + 4) {
+      debug("MP4: Atom 'trak.mdia.mdhd' is smaller than expected");
+      return;
+    }
+    const unsigned int unit   = data.toUInt(20U);
+    const unsigned int length = data.toUInt(24U);
+    d->length = unit ? length / unit : 0;
   }
 
   MP4::Atom *atom = trak->find("mdia", "minf", "stbl", "stsd");
@@ -110,11 +115,12 @@ MP4::Properties::Properties(File *file, MP4::Atoms *atoms, ReadStyle style)
   file->seek(atom->offset);
   data = file->readBlock(atom->length);
   if(data.mid(20, 4) == "mp4a") {
-    d->channels = data.mid(40, 2).toShort();
-    d->bitsPerSample = data.mid(42, 2).toShort();
-    d->sampleRate = data.mid(46, 4).toUInt();
+    d->codec         = AAC;
+    d->channels      = data.toShort(40U);
+    d->bitsPerSample = data.toShort(42U);
+    d->sampleRate    = data.toUInt(46U);
     if(data.mid(56, 4) == "esds" && data[64] == 0x03) {
-      long pos = 65;
+      uint pos = 65;
       if(data.mid(pos, 3) == "\x80\x80\x80") {
         pos += 3;
       }
@@ -125,10 +131,24 @@ MP4::Properties::Properties(File *file, MP4::Atoms *atoms, ReadStyle style)
           pos += 3;
         }
         pos += 10;
-        d->bitrate = (data.mid(pos, 4).toUInt() + 500) / 1000;
+        d->bitrate = (data.toUInt(pos) + 500) / 1000;
       }
     }
   }
+  else if (data.mid(20, 4) == "alac") {
+    if (atom->length == 88 && data.mid(56, 4) == "alac") {
+      d->codec         = ALAC;
+      d->bitsPerSample = data.at(69);
+      d->channels      = data.at(73);
+      d->bitrate       = data.toUInt(80U) / 1000;
+      d->sampleRate    = data.toUInt(84U);
+    }
+  }
+
+  MP4::Atom *drms = atom->find("drms");
+  if(drms) {
+    d->encrypted = true;
+  }
 }
 
 MP4::Properties::~Properties()
@@ -166,4 +186,14 @@ MP4::Properties::bitsPerSample() const
   return d->bitsPerSample;
 }
 
-#endif
+bool
+MP4::Properties::isEncrypted() const
+{
+  return d->encrypted;
+}
+
+MP4::Properties::Codec MP4::Properties::codec() const
+{
+  return d->codec;
+}
+
